from datetime import datetime
from typing import Callable, List, Optional

import click
from pycarlo.core import Client
from tabulate import tabulate

from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.tools import format_date, format_datetime


class AccountSecretsService:
    def __init__(
        self,
        config: Config,
        pycarlo_client: Client,
        command_name: str,
        print_func: Optional[Callable] = None,
    ):
        self._pycarlo_client = pycarlo_client
        self._command_name = command_name
        self._print_func = print_func or click.echo
        self._abort_on_error = True

    @manage_errors
    def create_secret(
        self,
        name: str,
        scope: str,
        value: str,
        description: Optional[str] = None,
        expires_at: Optional[datetime] = None,
    ):
        create_account_secret = """
            mutation createAccountSecret(
                $name: String!
                $value: String!
                $scope: String!
                $description: String
                $expiresAt: DateTime
            ) {
              createAccountSecret(
                name: $name
                value: $value
                scope: $scope
                description: $description
                expiresAt: $expiresAt
              ) {
                secret {
                  name
                }
              }
            }
        """
        response = self._pycarlo_client(
            create_account_secret,
            {
                "name": name,
                "value": value,
                "scope": scope,
                "description": description,
                "expiresAt": expires_at,
            },
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "account_secrets",
                "x-mcd-telemetry-command": self._command_name,
            },
        )

        secret = response.create_account_secret.secret  # type: ignore
        self._print_func(f"Created secret: '{secret.name}'.")

    @manage_errors
    def get_secret(self, name: str, reveal: bool = False):
        get_account_secret = """
            query getAccountSecret(
                $name: String!
                $reveal: Boolean!
            ) {
              getAccountSecret(
                name: $name
                reveal: $reveal
              ) {
                name
                scope
                description
                expiresAt
                value
                createdBy
                createdAt
                updatedBy
                lastUpdate
              }
            }
        """
        response = self._pycarlo_client(
            get_account_secret,
            {"name": name, "reveal": reveal},
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "account_secrets",
                "x-mcd-telemetry-command": self._command_name,
            },
        )

        secret = response.get_account_secret  # type: ignore
        if reveal:
            self._print_func(f"{secret.value}")
        else:
            self._print_func(f"Name: {secret.name}")
            self._print_func(f"Scope: {secret.scope}")
            self._print_func(f"Description: {secret.description or ''}")
            self._print_func(
                f"Expires at: {format_date(secret.expires_at) if secret.expires_at else 'never'}"  # type: ignore
            )
            self._print_func(f"Created by: {secret.created_by}")
            self._print_func(f"Created at: {format_datetime(secret.created_at)}")  # type: ignore
            self._print_func(f"Last updated by: {secret.updated_by}")
            self._print_func(f"Last updated at: {format_datetime(secret.last_update)}")  # type: ignore

    @manage_errors
    def delete_secret(self, name: str):
        delete_secret = """
            mutation deleteAccountSecret(
                $name: String!
            ) {
              deleteAccountSecret(
                name: $name
              ) {
                deleted
              }
            }
        """
        response = self._pycarlo_client(
            delete_secret,
            {"name": name},
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "account_secrets",
                "x-mcd-telemetry-command": self._command_name,
            },
        )

        deleted = response.delete_account_secret.deleted  # type: ignore
        if deleted:
            self._print_func(f"Deleted secret: '{name}'.")
        else:
            self._print_func(f"Secret '{name}' was not found.")

    @manage_errors
    def list_secrets(self):
        get_account_secrets = """
                query getAccountSecrets {
                  getAccountSecrets {
                    name
                    scope
                    description
                    expiresAt
                    createdBy
                    createdAt
                    updatedBy
                    lastUpdate
                  }
                }
            """
        response = self._pycarlo_client(
            get_account_secrets,
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "account_secrets",
                "x-mcd-telemetry-command": self._command_name,
            },
        )

        secrets = response.get_account_secrets  # type: ignore
        table: List = [
            [
                "Name",
                "Scope",
                "Description",
                "Expires at",
                "Created by",
                "Created at",
                "Last updated by",
                "Last updated at",
            ]
        ]
        for secret in secrets:
            table.append(
                [
                    secret.name,
                    secret.scope,
                    secret.description or "",
                    format_date(secret.expires_at) if secret.expires_at else "never",  # type: ignore
                    secret.created_by,
                    format_datetime(secret.created_at),  # type: ignore
                    secret.updated_by,
                    format_datetime(secret.last_update),  # type: ignore
                ]
            )
        click.echo(tabulate(table, headers="firstrow", tablefmt="fancy_grid", maxcolwidths=100))



from unittest import TestCase
from unittest.mock import Mock, patch

from pycarlo.core import Client, Session

from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.transactional.transactional_db import (
    TransactionalOnboardingService,
)
from montecarlodata.utils import GqlWrapper
from tests.test_base_onboarding import _SAMPLE_BASE_OPTIONS
from tests.test_common_user import _SAMPLE_CONFIG


class TransactionalOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._mc_client = Client(
            session=Session(
                endpoint=_SAMPLE_CONFIG.mcd_api_endpoint,
                mcd_id=_SAMPLE_CONFIG.mcd_id,
                mcd_token=_SAMPLE_CONFIG.mcd_token,
            )
        )
        self._service = TransactionalOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
            mc_client=self._mc_client,
            user_service=self._user_service_mock,
        )

    @patch.object(TransactionalOnboardingService, "test_new_credentials")
    def test_generic_transactional_db_flow(self, test_new_credentials_mock):
        expected_options = {
            **{
                "connection_type": "transactional-db",
                "warehouse_type": "transactional-db",
            },
            **_SAMPLE_BASE_OPTIONS,
        }

        test_new_credentials_mock.return_value = "tmp-string"

        self._service.onboard_transactional_db(**_SAMPLE_BASE_OPTIONS)
        test_new_credentials_mock.assert_called_once_with(**expected_options)



import click
from pycarlo.core import Client

from montecarlodata.collector.validation import CollectorValidationService
from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_ADD_CONNECTION_RESPONSE_FIELD,
    PINECONE_CONNECTION_TYPE,
    PINECONE_WAREHOUSE_TYPE,
)
from montecarlodata.queries.onboarding import (
    ADD_CONNECTION_MUTATION,
)


class VectorDbOnboardingService(BaseOnboardingService):
    def __init__(self, config: Config, mc_client: Client, command_name: str, **kwargs):
        super().__init__(config, command_name, **kwargs)
        self._validation_service = CollectorValidationService(
            config=config,
            mc_client=mc_client,
            user_service=self._user_service,
            request_wrapper=self._request_wrapper,
            command_name=self._command_name,
        )

    @manage_errors
    def onboard_pinecone(self, **kwargs) -> None:
        credentials_key = self._validation_service.test_new_credentials(
            connection_type=PINECONE_CONNECTION_TYPE,
            **kwargs,
        )

        environment = kwargs.get("environment")
        project_id = kwargs.get("project_id")
        name = kwargs.get("name") or f"{environment}:{project_id}"

        variables = {
            "connection_type": PINECONE_CONNECTION_TYPE,
            "create_warehouse_type": PINECONE_WAREHOUSE_TYPE,
            "dc_id": kwargs.get("dc_id"),
            "key": credentials_key,
            "name": name,
        }

        response = self._request_wrapper.make_request_v2(
            query=ADD_CONNECTION_MUTATION,
            operation=EXPECTED_ADD_CONNECTION_RESPONSE_FIELD,
            service="vector_db_onboarding_service",
            variables=variables,
        )

        conn_uuid = response.data.connection.uuid  # type: ignore
        click.echo(f"Successfully created connection (id={conn_uuid})")



import sys

import click

import montecarlodata.settings as settings
from montecarlodata.agents.commands import agents
from montecarlodata.collector.commands import collectors
from montecarlodata.common.user import UserService
from montecarlodata.config import ConfigManager
from montecarlodata.dataimport.commands import import_subcommand
from montecarlodata.discovery.commands import discovery
from montecarlodata.iac.commands import monitors
from montecarlodata.insights.commands import insights
from montecarlodata.integrations.commands import integrations
from montecarlodata.keys.commands import keys
from montecarlodata.management.commands import management
from montecarlodata.platform.commands import platform
from montecarlodata.secrets.commands import secrets
from montecarlodata.tools import dump_help


@click.group(help="Monte Carlo's CLI.")
@click.option(
    "--profile",
    default=settings.DEFAULT_PROFILE_NAME,
    help="Specify an MCD profile name. Uses default otherwise.",
)
@click.option(
    "--config-path",
    default=settings.DEFAULT_CONFIG_PATH,
    type=click.Path(dir_okay=True),
    help=(
        "Specify path where to look for config file. Uses "
        f"{settings.DEFAULT_CONFIG_PATH} otherwise."
    ),
)
@click.version_option()
@click.pass_context
def entry_point(ctx, profile, config_path):
    """
    Entry point for all subcommands and options. Reads configuration and sets as context,
    except when configuring or getting help.
    """
    if (
        ctx.invoked_subcommand != settings.CONFIG_SUB_COMMAND
        and settings.HELP_FLAG not in sys.argv[1:]
    ):
        config = ConfigManager(profile_name=profile, base_path=config_path).read()
        if not config:
            ctx.abort()
        ctx.obj = {"config": config}


@click.command(help="Configure the CLI.")
@click.option(
    "--profile-name",
    required=False,
    help="Specify a profile name for configuration.",
    default=settings.DEFAULT_PROFILE_NAME,
)
@click.option(
    "--config-path",
    required=False,
    help="Specify path where to look for config file.",
    default=settings.DEFAULT_CONFIG_PATH,
    type=click.Path(dir_okay=True),
)
@click.option("--mcd-id", prompt="Key ID", help="Monte Carlo token user ID.")
@click.option("--mcd-token", prompt="Secret", help="Monte Carlo token value.", hide_input=True)
def configure(profile_name, config_path, mcd_id, mcd_token):
    """
    Special subcommand for configuring the CLI
    """
    ConfigManager(profile_name=profile_name, base_path=config_path).write(
        mcd_id=mcd_id,
        mcd_token=mcd_token,
    )


@click.command(help="Validate that the CLI can Connect to Monte Carlo.")
@click.pass_obj
def validate(ctx):
    """
    Special subcommand for validating the CLI was correctly configured
    """
    first_name = UserService(
        config=ctx["config"],
        command_name="validate",
    ).user.first_name
    click.echo(f"Hi, {first_name}! All is well.")


@entry_point.command(help="Echo all help text.", name="help")
def echo_help():
    """
    Special subcommand to echo all help text.
    """
    dump_help(entry_point)


entry_point.add_command(integrations)
entry_point.add_command(configure)
entry_point.add_command(validate)
entry_point.add_command(collectors)
entry_point.add_command(discovery)
entry_point.add_command(monitors)
entry_point.add_command(import_subcommand)
entry_point.add_command(insights)
entry_point.add_command(management)
entry_point.add_command(agents)
entry_point.add_command(keys)
entry_point.add_command(secrets)
entry_point.add_command(platform)

# to allow this to be run as a script within an IDE (for debugging)
if __name__ == "__main__":
    entry_point()



from enum import Enum
from typing import Dict, List, Optional

import click
from tabulate import tabulate

from montecarlodata.common.data import ConnectionType
from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.errors import complain_and_abort, manage_errors
from montecarlodata.queries.keys import Queries
from montecarlodata.utils import GqlWrapper


class IntegrationKeyScope(Enum):
    Spark = "spark"
    DatabricksMetadata = "databricksmetadata"

    @classmethod
    def values(cls) -> List[str]:
        return list(map(lambda s: s.name, cls))


class IntegrationKeyService:
    _table_headers = ["Id", "Description", "Scope", "Created", "Created By"]

    def __init__(
        self,
        config: Config,
        command_name: str,
        gql: Optional[GqlWrapper] = None,
        user_service: Optional[UserService] = None,
    ):
        self._command_name = command_name
        self._gql = gql or GqlWrapper(config, command_name=self._command_name)
        self._user_service = user_service or UserService(
            config=config,
            command_name=self._command_name,
        )

        # used by @manage_errors decorator
        self._abort_on_error = True

    @manage_errors
    def create(
        self,
        description: Optional[str],
        scope: str,
        warehouse_ids: Optional[List[Optional[str]]] = None,
    ):
        response = self._gql.make_request_v2(
            query=Queries.create.query,
            operation=Queries.create.operation,
            service="integration_key_service",
            variables=self._resolve_variables(description, scope, warehouse_ids),
        )

        click.echo(f"Key id: {response.data.key.id}")  # type: ignore
        click.echo(f"Key secret: {response.data.key.secret}")  # type: ignore

    def _resolve_variables(
        self,
        description: Optional[str],
        scope: str,
        warehouse_ids: Optional[List[Optional[str]]] = None,
    ) -> Dict:
        variables = {"description": description, "scope": scope}

        if scope.lower() == IntegrationKeyScope.Spark.value:
            variables["warehouseIds"] = [self._resolve_lake_warehouse_id()]
        elif warehouse_ids:
            variables["warehouseIds"] = warehouse_ids

        return variables

    def _resolve_lake_warehouse_id(self) -> Optional[str]:
        lake_warehouse_ids = [
            w["uuid"]
            for w in self._user_service.warehouses
            if w["connectionType"] == ConnectionType.DataLake.value
        ]

        num_lakes = len(lake_warehouse_ids)
        if num_lakes == 0:
            complain_and_abort("Unable to resolve data lake connection: no lake connection found.")
        elif num_lakes > 1:
            complain_and_abort(
                "Unable to resolve data lake connection: multiple lake connections found."
            )
        else:
            return lake_warehouse_ids[0]

    @manage_errors
    def delete(self, key_id: str):
        response = self._gql.make_request_v2(
            query=Queries.delete.query,
            operation=Queries.delete.operation,
            service="integration_key_service",
            variables={"keyId": key_id},
        )

        if response.data.deleted:  # type: ignore
            click.echo("Key has been deleted.")
        else:
            click.echo("Key was not deleted.")

    @manage_errors
    def get_all(
        self,
        scope: Optional[str] = None,
        resource_uuid: Optional[str] = None,
        table_format: str = "fancy_grid",
    ):
        response = self._gql.make_request_v2(
            query=Queries.get_all.query,
            operation=Queries.get_all.operation,
            service="integration_key_service",
            variables={"scope": scope, "resourceUuid": resource_uuid},
        )

        data = [
            [
                key.id,
                key.description,
                key.scope,
                key.createdTime,
                f"{key.createdBy.firstName} {key.createdBy.lastName}",
            ]
            for key in response.data  # type: ignore
        ]

        table = tabulate(data, headers=self._table_headers, tablefmt=table_format)
        click.echo(table)



from unittest import TestCase
from unittest.mock import Mock, patch

from montecarlodata.common.data import ValidationResult
from montecarlodata.common.user import UserService
from montecarlodata.integrations.onboarding.etl.airflow import AirflowOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_ADD_ETL_CONNECTION_RESPONSE_FIELD,
    EXPECTED_TEST_AIRFLOW_RESPONSE_FIELD,
)
from montecarlodata.queries.onboarding import (
    ADD_ETL_CONNECTION_MUTATION,
    TEST_AIRFLOW_CRED_MUTATION,
)
from montecarlodata.utils import AwsClientWrapper, GqlWrapper
from tests.test_common_user import _SAMPLE_CONFIG


class AirflowOnboardingTest(TestCase):
    def setUp(self) -> None:
        self._user_service_mock = Mock(autospec=UserService)
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._service = AirflowOnboardingService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
            aws_wrapper=self._aws_wrapper_mock,
            user_service=self._user_service_mock,
        )
        self._service._disable_handle_errors = True

    @patch.object(AirflowOnboardingService, "_validate_connection")
    @patch("montecarlodata.integrations.onboarding.base.prompt_connection")
    def test_onboard_airflow(self, prompt_mock, validation_mock):
        validation_mock.return_value = ValidationResult(has_warnings=False, credentials_key="path")

        etl_name = "airflow_1"
        self._service.onboard_airflow(hostName="aws.com", etl_name=etl_name)

        validation_mock.assert_called_once_with(
            query=TEST_AIRFLOW_CRED_MUTATION,
            response_field=EXPECTED_TEST_AIRFLOW_RESPONSE_FIELD,
            hostName="aws.com",
        )

        self._request_wrapper_mock.make_request_v2.assert_called_with(
            query=ADD_ETL_CONNECTION_MUTATION,
            operation=EXPECTED_ADD_ETL_CONNECTION_RESPONSE_FIELD,
            service="onboarding_service",
            variables={
                "key": "path",
                "name": etl_name,
                "connectionType": "airflow",
            },
        )



import copy
from unittest import TestCase
from unittest.mock import Mock

import click

from montecarlodata.common.user import User, UserService
from montecarlodata.config import Config
from montecarlodata.queries.user import GET_USER_QUERY
from montecarlodata.utils import AwsClientWrapper, GqlWrapper

_SAMPLE_CONFIG = Config(
    mcd_id="1234",
    mcd_token="5678",
    mcd_api_endpoint="https://api.getmontecarlo.com/graphql",
)

_SAMPLE_USER = User(first_name="foo", last_name="bar")

_SAMPLE_DW_ID = "42"
_SAMPLE_USER_RESPONSE = {
    "getUser": {
        "firstName": _SAMPLE_USER.first_name,
        "lastName": _SAMPLE_USER.last_name,
        "account": {
            "uuid": "12345",
            "warehouses": [{"uuid": _SAMPLE_DW_ID}],
            "dataCollectors": [{"uuid": "5678", "stackArn": "arn", "active": True, "agents": []}],
        },
    }
}


class UserServiceTest(TestCase):
    def setUp(self) -> None:
        self._request_wrapper_mock = Mock(autospec=GqlWrapper)
        self._aws_wrapper_mock = Mock(autospec=AwsClientWrapper)

        self._request_wrapper_mock.make_request.return_value = _SAMPLE_USER_RESPONSE
        self._service = UserService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
        )
        self._request_wrapper_mock.reset_mock()

    def test_user(self):
        self.assertEqual(self._service.user.first_name, _SAMPLE_USER.first_name)
        self.assertEqual(self._service.user.last_name, _SAMPLE_USER.last_name)

    def test_initialization(self):
        user = UserService(
            _SAMPLE_CONFIG,
            command_name="test",
            request_wrapper=self._request_wrapper_mock,
        )
        self._request_wrapper_mock.make_request.assert_called_once_with(
            query=GET_USER_QUERY,
            service="user_service",
        )

        self.assertEqual(user._user, _SAMPLE_USER_RESPONSE)
        self.assertEqual(user.warehouses[0]["uuid"], _SAMPLE_DW_ID)

    def test_no_active_collectors(self):
        no_dc_user = copy.deepcopy(_SAMPLE_USER_RESPONSE)
        no_dc_user["getUser"]["account"]["dataCollectors"] = []

        self._request_wrapper_mock.make_request.return_value = no_dc_user
        with self.assertRaises(click.exceptions.Abort):
            UserService(
                _SAMPLE_CONFIG,
                command_name="test",
                request_wrapper=self._request_wrapper_mock,
            ).active_collector

    def test_get_collector(self):
        self.assertEqual(
            self._service.get_collector(),
            {"uuid": "5678", "stackArn": "arn", "active": True, "agents": []},
        )

    def test_get_collector_multi(self):
        multi_dc_user = copy.deepcopy(_SAMPLE_USER_RESPONSE)
        multi_dc_user["getUser"]["account"]["dataCollectors"] = [
            {"uuid": "1234", "stackArn": "arn", "active": True},
            {"uuid": "5678", "stackArn": "arn", "active": True},
        ]
        self._request_wrapper_mock.make_request.return_value = multi_dc_user

        with self.assertRaises(click.exceptions.Abort):
            UserService(
                _SAMPLE_CONFIG,
                command_name="test",
                request_wrapper=self._request_wrapper_mock,
            ).get_collector()

        self.assertEqual(
            UserService(
                _SAMPLE_CONFIG,
                command_name="test",
                request_wrapper=self._request_wrapper_mock,
            ).get_collector(dc_id="1234"),
            multi_dc_user["getUser"]["account"]["dataCollectors"][0],
        )

    def test_get_collector_with_no_collectors(self):
        no_dc_user = copy.deepcopy(_SAMPLE_USER_RESPONSE)
        no_dc_user["getUser"]["account"]["dataCollectors"] = []
        self._request_wrapper_mock.make_request.return_value = no_dc_user
        with self.assertRaises(click.exceptions.Abort):
            UserService(
                _SAMPLE_CONFIG,
                command_name="test",
                request_wrapper=self._request_wrapper_mock,
            ).get_collector()

    def test_resource_identifiers(self):
        self._request_wrapper_mock.make_request.return_value = {
            "getUser": {
                "firstName": _SAMPLE_USER.first_name,
                "lastName": _SAMPLE_USER.last_name,
                "account": {
                    "uuid": "12345",
                    "warehouses": [
                        {
                            "uuid": "d758a8a5-1c45-40a6-9ff2-9933616cd8b4",
                            "name": "wh-one",
                        }
                    ],
                    "bi": [
                        {
                            "uuid": "17c1c8be-271e-48c1-ae41-c6a3c8a84949",
                            "name": "bi-one",
                        }
                    ],
                    "dataCollectors": [{"uuid": "5678", "stackArn": "arn", "active": True}],
                },
            }
        }

        self.assertEqual(
            UserService(
                _SAMPLE_CONFIG,
                command_name="test",
                request_wrapper=self._request_wrapper_mock,
            ).resource_identifiers.get("wh-one"),
            "d758a8a5-1c45-40a6-9ff2-9933616cd8b4",
        )
        self.assertEqual(
            UserService(
                _SAMPLE_CONFIG,
                command_name="test",
                request_wrapper=self._request_wrapper_mock,
            ).resource_identifiers.get("d758a8a5-1c45-40a6-9ff2-9933616cd8b4"),
            "wh-one",
        )
        self.assertIsNone(
            UserService(
                _SAMPLE_CONFIG,
                command_name="test",
                request_wrapper=self._request_wrapper_mock,
            ).resource_identifiers.get("bi-one"),
        )
        self.assertIsNone(
            UserService(
                _SAMPLE_CONFIG,
                command_name="test",
                request_wrapper=self._request_wrapper_mock,
            ).resource_identifiers.get("17c1c8be-271e-48c1-ae41-c6a3c8a84949"),
        )



from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    ATHENA_CONNECTION_TYPE,
    EXPECTED_ATHENA_GQL_RESPONSE_FIELD,
    EXPECTED_GLUE_GQL_RESPONSE_FIELD,
    GLUE_CONNECTION_TYPE,
)
from montecarlodata.queries.onboarding import (
    TEST_ATHENA_CRED_MUTATION,
    TEST_GLUE_CRED_MUTATION,
)


class GlueAthenaOnboardingService(BaseOnboardingService):
    def __init__(self, config: Config, command_name: str, **kwargs):
        super().__init__(config, command_name=command_name, **kwargs)

    @manage_errors
    def onboard_glue(self, **kwargs) -> None:
        """
        Onboard a glue connection by validating and adding a connection
        """
        self.onboard(
            validation_query=TEST_GLUE_CRED_MUTATION,
            validation_response=EXPECTED_GLUE_GQL_RESPONSE_FIELD,
            connection_type=GLUE_CONNECTION_TYPE,
            **kwargs,
        )

    @manage_errors
    def onboard_athena(self, **kwargs) -> None:
        """
        Onboard an athena connection by validating and adding a connection
        """
        self.onboard(
            validation_query=TEST_ATHENA_CRED_MUTATION,
            validation_response=EXPECTED_ATHENA_GQL_RESPONSE_FIELD,
            connection_type=ATHENA_CONNECTION_TYPE,
            **kwargs,
        )



from configparser import ConfigParser
from uuid import uuid4

import pkg_resources
import pytest
from pytest_console_scripts import ScriptRunner
from pytest_mock import MockerFixture
from requests_mock import Mocker as RequestsMocker

from montecarlodata.settings import MCD_DEFAULT_API_ENDPOINT


@pytest.fixture(scope="session")
def mcd_id() -> str:
    return str(uuid4())


@pytest.fixture(scope="session")
def mcd_token() -> str:
    return "facedeadbeef"


@pytest.fixture(scope="session")
def montecarlo_version() -> str:
    return pkg_resources.get_distribution("montecarlodata").version


@pytest.fixture(autouse=True)
def use_mock_configuration(mcd_id: str, mcd_token: str, mocker: MockerFixture) -> None:
    def mock_parser() -> ConfigParser:
        info = {"mcd_id": mcd_id, "mcd_token": mcd_token}
        parser = ConfigParser()
        parser.add_section("default")
        for key, value in info.items():
            parser.set("default", key, value)
        return parser

    mocker.patch("montecarlodata.config.configparser.ConfigParser", new=mock_parser)


def test_version(script_runner: ScriptRunner, montecarlo_version: str) -> None:
    result = script_runner.run(["montecarlo", "--version"])

    assert result.returncode == 0
    assert result.stderr == ""
    assert f"montecarlo, version {montecarlo_version}" in result.stdout


def test_help(script_runner: ScriptRunner) -> None:
    result = script_runner.run(["montecarlo"])

    assert result.returncode == 0
    assert result.stderr == ""
    assert "Usage" in result.stdout
    assert "Monte Carlo's CLI." in result.stdout


def test_validate(script_runner: ScriptRunner, requests_mock: RequestsMocker) -> None:
    requests_mock.post(
        MCD_DEFAULT_API_ENDPOINT,
        json={
            "data": {
                "getUser": {
                    "firstName": "Test",
                    "lastName": "Runner",
                },
            },
        },
    )

    result = script_runner.run(["montecarlo", "validate"])

    assert result.returncode == 0
    assert result.stderr == ""
    assert result.stdout == "Hi, Test! All is well.\n"



import json
from typing import Any, Dict
from unittest import TestCase
from unittest.mock import patch

import click
import requests

import montecarlodata.settings as settings
from montecarlodata.common.data import MonolithResponse
from montecarlodata.config import Config
from montecarlodata.utils import GqlWrapper

_SAMPLE_ID = "1234"
_SAMPLE_TOKEN = "5678"
_SAMPLE_ENDPOINT = "graphql"
_SAMPLE_QUERY = "foo"
_SAMPLE_SERVICE = "test"
_SAMPLE_VARIABLES = {"bar": "qux"}
_SAMPLE_RESPONSE = {"data": "foo"}
_SAMPLE_ERRORS = {"errors": "error"}
_SAMPLE_CONFIG = Config(
    mcd_id=_SAMPLE_ID, mcd_token=_SAMPLE_TOKEN, mcd_api_endpoint=_SAMPLE_ENDPOINT
)

_SAMPLE_GQL_RAW_VARS = {"bar_qux": "foo"}
_SAMPLE_GQL_TRANSFORMED_VARS = {"barQux": "foo"}
_SAMPLE_GQL_OP_NAME = "operation"
_SAMPLE_GQL_DATA = {"foo": "bar"}
_SAMPLE_GOOD_GQL_RESPONSE = {"data": {_SAMPLE_GQL_OP_NAME: _SAMPLE_GQL_DATA}}
_SAMPLE_BAD_GQL_RESPONSE = {"errors": [{"message": "API is broken"}]}


class MockGoodRequest:
    text = json.dumps(_SAMPLE_RESPONSE)

    @staticmethod
    def raise_for_status():
        return False


class MockBadRequest:
    text = json.dumps(_SAMPLE_RESPONSE)

    @staticmethod
    def raise_for_status():
        raise requests.exceptions.HTTPError


class MockGqlErrors(MockGoodRequest):
    text = json.dumps({**_SAMPLE_RESPONSE, **_SAMPLE_ERRORS})


class GqlUtilTest(TestCase):
    def setUp(self) -> None:
        self._service = GqlWrapper(_SAMPLE_CONFIG, command_name="test")

    def tearDown(self) -> None:
        self._service._abort_on_error = True
        settings.MCD_USER_ID_HEADER = None

    @patch.object(GqlWrapper, "_post", return_value=MockGoodRequest)
    def test_make_request(self, request_mock):
        self.assertEqual(
            self._service.make_request(
                query=_SAMPLE_QUERY,
                service=_SAMPLE_SERVICE,
                variables=_SAMPLE_VARIABLES,
            ),
            _SAMPLE_RESPONSE["data"],
        )
        request_mock.assert_called_once_with(**self._build_request())

    @patch.object(GqlWrapper, "_post", return_value=MockBadRequest)
    def test_request_with_status(self, request_mock):
        self._test_error_flow(request_mock)

    @patch.object(GqlWrapper, "_post", side_effect=requests.exceptions.ConnectionError)
    def test_request_with_connection_error(self, request_mock):
        self._test_error_flow(request_mock)

    @patch.object(GqlWrapper, "_post", side_effect=requests.exceptions.Timeout)
    def test_request_with_timeout(self, request_mock):
        self._test_error_flow(request_mock)

    @patch.object(GqlWrapper, "_post", return_value=MockGqlErrors)
    def test_request_with_gql_errors(self, request_mock):
        self._test_error_flow(request_mock)

    @patch.object(GqlWrapper, "_post", return_value=MockGoodRequest)
    def test_make_request_with_user_id(self, request_mock):
        user_id = "thor"
        settings.MCD_USER_ID_HEADER = user_id
        self.assertEqual(
            self._service.make_request(
                query=_SAMPLE_QUERY,
                service=_SAMPLE_SERVICE,
                variables=_SAMPLE_VARIABLES,
            ),
            _SAMPLE_RESPONSE["data"],
        )
        request_mock.assert_called_once_with(
            headers={"user-id": user_id},
            payload={
                "query": _SAMPLE_QUERY,
                "variables": _SAMPLE_VARIABLES,
            },
        )

    @patch.object(GqlWrapper, "_make_request", return_value=_SAMPLE_GOOD_GQL_RESPONSE)
    def test_make_request_v2(self, request_mock):
        self.assertEqual(
            self._service.make_request_v2(
                query=_SAMPLE_QUERY,
                operation=_SAMPLE_GQL_OP_NAME,
                service=_SAMPLE_SERVICE,
                variables=_SAMPLE_GQL_RAW_VARS,
            ),
            MonolithResponse(data=_SAMPLE_GQL_DATA, errors=None),
        )
        request_mock.assert_called_once_with(
            query=_SAMPLE_QUERY,
            service=_SAMPLE_SERVICE,
            variables=_SAMPLE_GQL_TRANSFORMED_VARS,
        )

    @patch.object(GqlWrapper, "_make_request", return_value=_SAMPLE_BAD_GQL_RESPONSE)
    def test_make_request_v2_with_errors(self, request_mock):
        with self.assertRaises(click.exceptions.Abort):
            self._service.make_request_v2(
                query=_SAMPLE_QUERY,
                operation=_SAMPLE_GQL_OP_NAME,
                service=_SAMPLE_SERVICE,
                variables=_SAMPLE_GQL_RAW_VARS,
            )
            request_mock.assert_called_once_with(
                query=_SAMPLE_QUERY,
                service=_SAMPLE_SERVICE,
                variables=_SAMPLE_GQL_TRANSFORMED_VARS,
            )

    @patch.object(GqlWrapper, "_make_request", return_value=_SAMPLE_BAD_GQL_RESPONSE)
    def test_make_request_v2_with_skipped_errors(self, request_mock):
        self._service._abort_on_error = False
        self.assertEqual(
            self._service.make_request_v2(
                query=_SAMPLE_QUERY,
                operation=_SAMPLE_GQL_OP_NAME,
                service=_SAMPLE_SERVICE,
                variables=_SAMPLE_GQL_RAW_VARS,
            ),
            MonolithResponse(data=None, errors=_SAMPLE_BAD_GQL_RESPONSE["errors"]),
        )
        request_mock.assert_called_once_with(
            query=_SAMPLE_QUERY,
            service=_SAMPLE_SERVICE,
            variables=_SAMPLE_GQL_TRANSFORMED_VARS,
        )

    def _test_error_flow(self, request_mock: Any):
        with self.assertRaises(click.exceptions.Abort):
            self._service.make_request(
                query=_SAMPLE_QUERY,
                service=_SAMPLE_SERVICE,
                variables=_SAMPLE_VARIABLES,
            )
        request_mock.assert_called_once_with(**self._build_request())

    def test_convert_snakes_to_camels(self):
        snakes = {
            "foo_bar": "test1",
            "qux": "test2",
            "baz_quux_quuz": "test3",
            "baz": {"quux_quuz": "test4"},
        }
        expected_camel = {
            "bazQuuxQuuz": "test3",
            "fooBar": "test1",
            "qux": "test2",
            "baz": {"quuxQuuz": "test4"},
        }
        self.assertDictEqual(self._service.convert_snakes_to_camels(snakes), expected_camel)

    @staticmethod
    def _build_request(
        *,
        query: str = _SAMPLE_QUERY,
        service: str = _SAMPLE_SERVICE,
        variables: Dict = _SAMPLE_VARIABLES,
    ):
        return {
            "headers": {
                "x-mcd-id": _SAMPLE_ID,
                "x-mcd-token": _SAMPLE_TOKEN,
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": service,
                "x-mcd-telemetry-command": "test",
                "Content-Type": "application/json",
            },
            "payload": {
                "query": query,
                "variables": variables,
            },
        }



import time
from dataclasses import dataclass
from typing import Dict, List, Optional
from uuid import UUID

import click
from box import Box
from dataclasses_json import DataClassJsonMixin
from pycarlo.common.errors import GqlError
from pycarlo.core import Client
from retry.api import retry_call
from tabulate import tabulate

from montecarlodata.queries.platform import (
    MUTATION_TRIGGER_CAAS_MIGRATION_TEST,
    QUERY_CAAS_MIGRATION_TEST_STATUS,
    QUERY_GET_SERVICES,
)


class CaasMigrationNotReadyError(Exception):
    pass


@dataclass
class MigrationTestedConnection(DataClassJsonMixin):
    success: bool
    name: Optional[str]
    uuid: str
    connection_type: str
    connection_subtype: Optional[str] = None
    error_message: Optional[str] = None
    suggested_action: Optional[str] = None


@dataclass
class MigrationTestedAgent(DataClassJsonMixin):
    success: bool
    agent_type: str
    platform: Optional[str]
    endpoint: str
    uuid: str
    error_message: Optional[str] = None
    suggested_action: Optional[str] = None


@dataclass
class MigrationValidationsResult(DataClassJsonMixin):
    success: bool
    tested_agent: Optional[MigrationTestedAgent] = None
    tested_connections: Optional[List[MigrationTestedConnection]] = None
    error_message: Optional[str] = None


class PlatformService:
    def __init__(
        self,
        mc_client: Client,
        command_name: str,
    ):
        self._mc_client = mc_client
        self._command_name = command_name

    def caas_migration_test(self, dc_id: Optional[UUID]):
        dc_id = self._disambiguate_dc(dc_id)
        try:
            operation = self._mc_client(
                MUTATION_TRIGGER_CAAS_MIGRATION_TEST,
                variables={"dcUuid": str(dc_id) if dc_id else None},
                additional_headers={
                    "x-mcd-telemetry-reason": "cli",
                    "x-mcd-telemetry-service": "platform_service",
                    "x-mcd-telemetry-command": self._command_name,
                },
            )
        except GqlError as e:
            click.echo(str(e))
            raise click.Abort()

        migration_uuid = operation.trigger_platform_migration_test.migration_uuid  # type: ignore
        click.echo(
            "Migration test triggered, this might take a few minutes, waiting for result ..."
        )
        time.sleep(10)  # wait to start polling
        try:
            output = retry_call(
                self._get_caas_migration_test_status,
                fargs=[dc_id, migration_uuid],
                tries=20,
                delay=18,
                exceptions=CaasMigrationNotReadyError,
            )
        except CaasMigrationNotReadyError:
            click.echo("Migration test timed out.")
            raise click.Abort()
        except GqlError as e:
            click.echo(str(e))
            raise click.Abort()

        self._echo_caas_migration_test_status(output)

    def list_services(self):
        data_collectors = self._get_active_data_collectors()
        table: List[List[str]] = []
        for dc in data_collectors:
            agent = self._get_active_agent(dc.agents)
            table.append(
                [
                    dc.uuid,
                    dc.deployment_type,
                    (
                        agent.endpoint
                        if agent
                        else (dc.stack_arn if dc.deployment_type == "REMOTE_V1" else "N/A")
                    ),
                ]
            )
        if not table:
            click.echo("No services found")
            raise click.Abort()
        headers = [
            "Service ID",
            "Deployment Type",
            "Endpoint",
        ]
        click.echo(tabulate(table, headers=headers, tablefmt="fancy_grid"))

    @staticmethod
    def _get_active_agent(agents: List) -> Optional[Box]:
        return next(filter(lambda a: not a.is_deleted, agents), None)

    @classmethod
    def _echo_caas_migration_test_status(cls, output: Dict):
        result = MigrationValidationsResult.from_dict(output)
        success = result.success
        error_message = result.error_message
        if not success and error_message:
            click.echo(f"Migration test failed: {error_message}")
            raise click.Abort()

        tested_agent = result.tested_agent
        if tested_agent:
            cls._echo_caas_migration_agent_result(tested_agent)
            if tested_agent.agent_type == "REMOTE_AGENT":
                return
        table: List[List[str]] = []
        for connection in result.tested_connections:  # type: ignore
            table.append(
                [
                    connection.name,  # type: ignore
                    connection.uuid,
                    connection.connection_subtype or connection.connection_type,
                    "Success" if connection.success else "Failed",
                    ("None" if connection.success else connection.suggested_action or ""),
                    connection.error_message[:200] if connection.error_message else "",
                ]
            )
        if not table:
            click.echo("No connections tested")
            raise click.Abort()

        headers = [
            "Name",
            "Connection UUID",
            "Connection Type",
            "Result",
            "Required Action",
            "Error Message",
        ]
        widths = [20, 36, 20, 7, 40, 40]
        click.echo(tabulate(table, headers=headers, tablefmt="fancy_grid", maxcolwidths=widths))

    @staticmethod
    def _echo_caas_migration_agent_result(tested_agent: MigrationTestedAgent):
        table = [
            [
                tested_agent.agent_type,
                tested_agent.platform or "",
                tested_agent.endpoint,
                "Success" if tested_agent.success else "Failed",
                ("None" if tested_agent.success else tested_agent.suggested_action or ""),
                tested_agent.error_message[:200] if tested_agent.error_message else "",
            ]
        ]
        headers = [
            "Agent Type",
            "Platform",
            "Endpoint",
            "Result",
            "Required Action",
            "Error Message",
        ]
        widths = [16, 8, 32, 7, 40, 40]
        click.echo(tabulate(table, headers=headers, tablefmt="fancy_grid", maxcolwidths=widths))

    def _get_caas_migration_test_status(self, dc_id: Optional[UUID], migration_uuid: UUID) -> Dict:
        operation = self._mc_client(
            QUERY_CAAS_MIGRATION_TEST_STATUS,
            variables={
                "dcUuid": str(dc_id) if dc_id else None,
                "migrationUuid": str(migration_uuid),
            },
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "platform_service",
                "x-mcd-telemetry-command": self._command_name,
            },
        )
        migration_output = operation.get_platform_migration_status.output  # type: ignore
        if not migration_output or migration_output.get("status") != "dry_run_completed":  # type: ignore
            raise CaasMigrationNotReadyError()
        return migration_output  # type: ignore

    def _get_active_data_collectors(self) -> List[Box]:
        operation = self._mc_client(
            QUERY_GET_SERVICES,
            additional_headers={
                "x-mcd-telemetry-reason": "cli",
                "x-mcd-telemetry-service": "platform_service",
                "x-mcd-telemetry-command": self._command_name,
            },
        )
        data_collectors = operation.get_user.account.data_collectors  # type: ignore
        return list(filter(lambda dc: dc.active, data_collectors))  # type: ignore

    def _disambiguate_dc(self, dc_id: Optional[UUID]) -> UUID:
        active_dcs = self._get_active_data_collectors()
        if dc_id:
            dc = next((dc for dc in active_dcs if dc.uuid == str(dc_id)), None)
            if not dc:
                click.echo(
                    "No active service found with the specified ID, use "
                    "`montecarlo platform list` to list the active services."
                )
                raise click.Abort()
        else:
            active_dc_count = len(active_dcs)
            if active_dc_count == 0:
                click.echo("There are no active services in this account.")
                raise click.Abort()
            elif active_dc_count > 1:
                click.echo(
                    "There are multiple active services, please specify one using --service-id."
                )
                raise click.Abort()
            dc = active_dcs[0]
        if dc.deployment_type in ("CLOUD_V2", "REMOTE_V2"):
            click.echo(f"No need to migrate {dc.deployment_type} services.")
            raise click.Abort()
        return UUID(dc.uuid)



from typing import Dict, Optional

from montecarlodata.common.user import UserService
from montecarlodata.config import Config
from montecarlodata.errors import complain_and_abort


class AirflowService:
    def __init__(
        self,
        config: Config,
        command_name: str,
        user_service: Optional[UserService] = None,
    ):
        self._command_name = command_name
        self._user_service = user_service or UserService(
            config=config,
            command_name=self._command_name,
        )

    def get_airflow_resource_uuid(self, name: Optional[str] = None) -> Optional[str]:
        etl_containers = self._user_service.etl_containers or []
        airflow_containers = [
            container for container in etl_containers if self._is_airflow_container(container, name)
        ]
        if not airflow_containers:
            if name:
                complain_and_abort(f"No Airflow connection found with name {name}")
            else:
                complain_and_abort("No Airflow connection found")
            return None
        elif len(airflow_containers) > 1:
            if name:
                complain_and_abort(f"Multiple Airflow connections found with name {name}")
            else:
                complain_and_abort("Multiple Airflow connections found, use --name to disambiguate")
            return None
        return airflow_containers[0].get("uuid")

    @staticmethod
    def _is_airflow_container(container: Dict, name: Optional[str]) -> bool:
        if name and name != container["name"]:
            return False
        connections = container.get("connections", [])
        return any(connection for connection in connections if connection["type"] == "AIRFLOW")



from pycarlo.core import Client

from montecarlodata.collector.validation import CollectorValidationService
from montecarlodata.config import Config
from montecarlodata.errors import manage_errors
from montecarlodata.integrations.onboarding.base import BaseOnboardingService
from montecarlodata.integrations.onboarding.fields import (
    EXPECTED_ADD_ETL_CONNECTION_RESPONSE_FIELD,
    INFORMATICA_CONNECTION_TYPE,
)
from montecarlodata.queries.onboarding import ADD_ETL_CONNECTION_MUTATION


class InformaticaOnboardingService(BaseOnboardingService):
    def __init__(self, config: Config, mc_client: Client, command_name: str, **kwargs):
        super().__init__(config, command_name, **kwargs)
        self._validation_service = CollectorValidationService(
            config=config,
            mc_client=mc_client,
            user_service=self._user_service,
            request_wrapper=self._request_wrapper,
            command_name=self._command_name,
        )

    @manage_errors
    def onboard(self, **kwargs) -> None:
        # run validations and store credentials (if passed)
        key = self._validation_service.test_new_credentials(
            connection_type=INFORMATICA_CONNECTION_TYPE,
            **kwargs,
        )
        if key:
            # create ETL connection
            self.add_connection(
                connection_type=INFORMATICA_CONNECTION_TYPE,
                connection_query=ADD_ETL_CONNECTION_MUTATION,
                connection_response=EXPECTED_ADD_ETL_CONNECTION_RESPONSE_FIELD,
                etl_name=kwargs.pop("name"),
                key=key,
                **kwargs,
            )
